/*
 * Ambient - the ultimate desktop
 * ------------------------------
 * Copyright 2001-2005 by David Gerber <zapek@morphos.net>
 * All Rights Reserved
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 * $Id: panelitem_listclass.c,v 1.9 2008/05/10 17:35:30 kiero Exp $
 */

#include "ambient.h"

/* public */

/* private */
#include "mui_func.h"
#include "panelitem.h"
#include "gfx_bitmap.h"


struct Data {
	int dummy;
};


static APTR create_image(APTR obj, APTR bm)
{
	APTR iobj;

	if ((iobj = BitmapObject,
		MUIA_Bitmap_Bitmap, gfx_bitmap_bm(bm),
		MUIA_Bitmap_Height, PANELIMAGE_HEIGHT, 
		MUIA_Bitmap_Width,  PANELIMAGE_WIDTH,
		MUIA_FixHeight,     PANELIMAGE_HEIGHT, 
		MUIA_FixWidth,      PANELIMAGE_WIDTH,
		MUIA_Bitmap_Alpha,  0xffffffff,
		End))
	{
		return ((APTR)DoMethod(obj, MUIM_List_CreateImage, iobj, 0));
	}

	return (NULL);
}


DEFMMETHOD(List_Display)
{
	static TEXT tmp[40];
	struct itemlist_entry *ie = msg->entry;

	if (!ie->image)
	{
		ASSERT(ie->bm);
		ie->image = create_image(obj, ie->bm);
	}

	if (ie->image)
	{
		snprintf(tmp, sizeof(tmp), "\033O[%08lx] %s", (ULONG)ie->image, ie->name);
	}
	else
	{
		snprintf(tmp, sizeof(tmp), "%s", ie->name);
	}

	msg->array[0] = tmp;

	return (0);
}


DEFMMETHOD(List_Destruct)
{
	delete_panelitem((struct itemlist_entry *)msg->entry);

	return (0);
}


DEFNEW
{
	struct Data *data;

	obj = DoSuperNew(cl, obj,
		InputListFrame,
		MUIA_List_MinLineHeight, PANELIMAGE_HEIGHT,
		MUIA_List_AdjustWidth,   TRUE,
		TAG_MORE, INITTAGS);

	if (!obj)
	{
		return (NULL);
	}

	data = INST_DATA(cl, obj);

	/*  Add buttons and menus. zaplets are loaded
	 *  from disk later (XXX: add menus one day)
	 */
	{
		LONG pti;

		for(pti = 0; pti < PT_END; pti++)
		{
			struct itemlist_entry *ie = create_panelitem(pti, NULL);

			/* insert only if we succeded. some values might be 'empty' */

			if (ie != NULL)
			{
				DoMethod(obj, MUIM_List_InsertSingle, ie, MUIV_List_Insert_Bottom);
			}
		}
	}
	
	return ((ULONG)obj);
}


DEFGET
{
	switch (msg->opg_AttrID)
	{
		case MA_Panelitem_list_IsList:
			*msg->opg_Storage = TRUE;
			return (TRUE);
	}
	return (DOSUPER);
}


DEFMMETHOD(Setup)
{
	ULONG i;
	struct itemlist_entry *ie;

	if (!DOSUPER)
	{
		return (FALSE);
	}
	
	for (i = 0; ; i++)
	{
		DoMethod(obj, MUIM_List_GetEntry, i, &ie);

		if (ie)
		{
			if (!ie->image)
			{
				ASSERT(ie->bm);
				ie->image = create_image(obj, ie->bm);
			}
		}
		else
		{
			break;
		}
	}

	return (TRUE);
}


DEFMMETHOD(Cleanup)
{
	ULONG i;
	struct itemlist_entry *ie;

	for (i = 0; ; i++)
	{
		DoMethod(obj, MUIM_List_GetEntry, i, &ie);

		if (ie)
		{
			if (ie->image)
			{
				DoMethod(obj, MUIM_List_DeleteImage, ie->image);
				ie->image = NULL;
			}
		}
		else
		{
			break;
		}
	}

	return (DOSUPER);
}


BEGINMTABLE
DECNEW
DECGET
DECMMETHOD(List_Display)
DECMMETHOD(List_Destruct)
DECMMETHOD(Setup)
DECMMETHOD(Cleanup)
ENDMTABLE

DECSUBCLASS_NC(MUIC_List, panelitem_listclass)
